use project_euler::number::Number;

fn main() {
    let fib_idx = find_fib(1000);
    println!("{fib_idx}");
}

// Way too slow, from 20 onward we go from µs to ms and from 32 we measure in seconds
//fn fib_recursion(n: Number) -> usize {
//    let one = Number::from(1);
//    if n == one {
//        return 1;
//    }
//    let two = Number::from(2);
//    if n == two {
//        return 1;
//    }
//    fib_recursion(n.clone() - two) + fib_recursion(n - one)
//}

// almost as fast as the no recursion solution but more complicated and more memory intensiv
// creating an inner function to not have to pass in the hashmap from outside makes it slightly slower
//fn fib_recursion_cached(n: Number) -> Number {
//    let mut cache: HashMap<Number, Number> = HashMap::new();
//    fn inner_fib_recursion_cached(n: Number, cache: &mut HashMap<Number, Number>) -> Number {
//        let one = Number::from(1);
//        if n == one {
//            return one;
//        }
//        let two = Number::from(2);
//        if n == two {
//            return one;
//        }
//        match cache.get(&n) {
//            Some(res) => return res.clone(),
//            None => {
//                let res = inner_fib_recursion_cached(n.clone() - two, cache) + inner_fib_recursion_cached(n.clone() - one, cache);
//                cache.insert(n.clone(), res.clone());
//                return res;
//            },
//        }
//    }
//    inner_fib_recursion_cached(n, &mut cache)
//}

// faster than recursion
fn fib(n: usize) -> Number {
    let mut last_two = (Number::from(1), Number::from(1));
    let mut iteration = 1;
    while iteration < n {
        last_two = (last_two.1.clone(), last_two.0 + last_two.1);
        iteration += 1;
    }

    last_two.0
}

fn find_fib(num_digits: usize) -> usize {
    let mut n = 1;
    loop {
        let fib = fib(n);
        let fib_len = fib.digits.len();
        if fib_len == num_digits {
            return n;
        }
        n += 1;
    }
}